@@ -236,7 +236,7 @@ main ()
                              NULL, MHD_OPTION_END);
   if (NULL == daemon)
     return 1;
-  getchar ();
+  (void) getchar ();
   MHD_stop_daemon (daemon);
   return 0;
 }
