@@ -666,7 +666,7 @@ build_header_response (struct MHD_Connection *connection)
   int must_add_content_length;
 
   EXTRA_CHECK (NULL != connection->version);
-  if (0 == strlen (connection->version))
+  if (0 == connection->version[0])
     {
       data = MHD_pool_allocate (connection->pool, 0, MHD_YES);
       connection->write_buffer = data;
@@ -1796,7 +1796,7 @@ process_broken_line (struct MHD_Connection *connection,
       return MHD_NO;
     }
   /* we still have the current line to deal with... */
-  if (0 != strlen (line))
+  if (0 != line[0])
     {
       if (MHD_NO == process_header_line (connection, line))
         {
@@ -2227,7 +2227,11 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
         {
         case MHD_CONNECTION_INIT:
           line = get_next_header_line (connection);
-          if (NULL == line)
+          /* Check for empty string, as we might want
+             to tolerate 'spurious' empty lines; also
+             NULL means we didn't get a full line yet. */
+          if ( (NULL == line) ||
+               (0 == line[0]) )
             {
               if (MHD_CONNECTION_INIT != connection->state)
                 continue;
@@ -2258,7 +2262,7 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
                 }
               break;
             }
-          if (strlen (line) == 0)
+          if (0 == line[0])
             {
               connection->state = MHD_CONNECTION_HEADERS_RECEIVED;
               continue;
@@ -2289,7 +2293,7 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
           if (MHD_NO ==
               process_broken_line (connection, line, MHD_HEADER_KIND))
             continue;
-          if (0 == strlen (line))
+          if (0 == line[0])
             {
               connection->state = MHD_CONNECTION_HEADERS_RECEIVED;
               continue;
@@ -2366,7 +2370,7 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
                 }
               break;
             }
-          if (0 == strlen (line))
+          if (0 == line[0])
             {
               connection->state = MHD_CONNECTION_FOOTERS_RECEIVED;
               continue;
@@ -2397,7 +2401,7 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
           if (MHD_NO ==
               process_broken_line (connection, line, MHD_FOOTER_KIND))
             continue;
-          if (0 == strlen (line))
+          if (0 == line[0])
             {
               connection->state = MHD_CONNECTION_FOOTERS_RECEIVED;
               continue;
@@ -2555,10 +2559,15 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
               /* can try to keep-alive */
               connection->version = NULL;
               connection->state = MHD_CONNECTION_INIT;
+              /* Reset the read buffer to the starting size,
+                 preserving the bytes we have already read. */
               connection->read_buffer
                 = MHD_pool_reset (connection->pool,
                                   connection->read_buffer,
-                                  connection->read_buffer_offset);
+                                  connection->read_buffer_offset,
+                                  connection->daemon->pool_size / 2);
+              connection->read_buffer_size
+                = connection->daemon->pool_size / 2;
             }
 	  connection->client_aware = MHD_NO;
           connection->client_context = NULL;
